@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 # ########################################################################
-# Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -42,10 +42,12 @@ function display_help()
   echo "    [-r]--relocatable] create a package to support relocatable ROCm"
   echo "    [-g|--debug] -DCMAKE_BUILD_TYPE=Debug (default is =Release)"
   echo "    [-k|--relwithdebinfo] -DCMAKE_BUILD_TYPE=RelWithDebInfo"
-  echo "    [--hip-clang] build library for amdgpu backend using hip-clang"
+  echo "    [--hip-clang] build library for amdgpu backend using amdclang"
   echo "    [--static] build static library"
   echo "    [--address-sanitizer] build with address sanitizer"
   echo "    [--codecoverage] build with code coverage profiling enabled"
+  echo "    [--gprof] enable profiling functionality with GNU gprof"
+  echo "    [--keep-build-tmp] do not remove the temporary build artifacts or build_tmp"
 }
 
 # This function is helpful for dockerfiles that do not have sudo installed, but the default user is root
@@ -59,10 +61,10 @@ supported_distro( )
   fi
 
   case "${ID}" in
-    ubuntu|centos|rhel|fedora|sles|opensuse-leap)
+    ubuntu|centos|rhel|fedora|sles|opensuse-leap|mariner|azurelinux)
         true
         ;;
-    *)  printf "This script is currently supported on Ubuntu, CentOS, RHEL, Fedora and SLES\n"
+    *)  printf "This script is currently supported on Ubuntu, CentOS, RHEL, Fedora, SLES, mariner and azurelinux\n"
         exit 2
         ;;
   esac
@@ -159,12 +161,14 @@ install_packages( )
   local library_dependencies_centos8=( "epel-release" "make" "gcc-c++" "rpm-build" "numactl-libs" )
   local library_dependencies_fedora=( "gcc-gfortran" "make" "gcc-c++" "libcxx-devel" "rpm-build" "numactl-libs" )
   local library_dependencies_sles=( "gcc-fortran" "make" "gcc-c++" "libcxxtools9" "rpm-build" )
+  local library_dependencies_mariner=( "gfortran" "make" "rpm-build" )
 
   local client_dependencies_ubuntu=( "python3" "python3-yaml")
   local client_dependencies_centos=( "python36" "python3-pip" )
   local client_dependencies_centos8=( "python39" "python3-virtualenv" )
   local client_dependencies_fedora=( "python36" "PyYAML" "python3-pip" )
   local client_dependencies_sles=( "pkg-config" "dpkg" "python3-pip" )
+  local client_dependencies_mariner=( "python3" "python3-yaml" )
 
   if [[ "${tensile_msgpack_backend}" == true ]]; then
     library_dependencies_ubuntu+=("libmsgpack-dev")
@@ -240,6 +244,24 @@ install_packages( )
       fi
       ;;
 
+    mariner)
+#     yum -y update brings *all* installed packages up to date
+#     without seeking user approval
+      install_yum_packages "${library_dependencies_mariner[@]}"
+      if [[ "${build_clients}" == true ]]; then
+        install_yum_packages "${client_dependencies_mariner[@]}"
+        pip3 install pyyaml
+      fi
+      ;;
+
+    azurelinux)
+      install_dnf_packages "${library_dependencies_mariner[@]}"
+      if [[ "${build_clients}" == true ]]; then
+        install_dnf_packages "${client_dependencies_mariner[@]}"
+        pip3 install pyyaml
+      fi
+      ;;
+
     fedora)
 #     elevate_if_not_root dnf -y update
       install_dnf_packages "${library_dependencies_fedora[@]}"
@@ -260,7 +282,7 @@ install_packages( )
       fi
       ;;
     *)
-      echo "This script is currently supported on Ubuntu, CentOS, RHEL and Fedora"
+      echo "This script is currently supported on Ubuntu, CentOS, RHEL, Fedora and Mariner"
       exit 2
       ;;
   esac
@@ -359,6 +381,7 @@ matrices_dir_install=
 gpu_architecture=all
 cpu_ref_lib=blis
 tensile_cov=
+tensile_threads=$(nproc)
 tensile_fork=
 tensile_merge_files=
 tensile_tag=
@@ -367,6 +390,8 @@ tensile_version=
 build_tensile=true
 tensile_msgpack_backend=true
 update_cmake=true
+enable_gprof=false
+keep_build_tmp=false
 
 
 rocm_path=/opt/rocm
@@ -381,7 +406,7 @@ fi
 # check if we have a modern version of getopt that can handle whitespace and long parameters
 getopt -T
 if [[ $? -eq 4 ]]; then
-  GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,clients,dependencies,debug,hip-clang,static,relocatable,codecoverage,relwithdebinfo,address-sanitizer,merge-files,no-merge-files,no_tensile,no-tensile,msgpack,no-msgpack,logic:,cov:,fork:,branch:,test_local_path:,cpu_ref_lib:,build_dir:,use-custom-version:,architecture: --options hicdgrka:o:l:f:b:nu:t: -- "$@")
+  GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,clients,dependencies,debug,hip-clang,static,relocatable,codecoverage,relwithdebinfo,address-sanitizer,merge-files,no-merge-files,no_tensile,no-tensile,msgpack,no-msgpack,logic:,cov:,fork:,branch:,test_local_path:,cpu_ref_lib:,build_dir:,use-custom-version:,architecture:,gprof,keep-build-tmp,legacy_hipblas_direct --options hicdgrka:j:o:l:f:b:nu:t: -- "$@")
 else
   echo "Need a new version of getopt"
   exit 1
@@ -449,6 +474,9 @@ while true; do
         -f|--fork)
             tensile_fork=${2}
             shift 2 ;;
+        -j)
+            tensile_threads=${2}
+            shift 2;;
         -b|--branch)
             tensile_tag=${2}
             shift 2 ;;
@@ -480,6 +508,15 @@ while true; do
         --cmake_install)
             update_cmake=true
             shift ;;
+        --gprof)
+            enable_gprof=true
+            shift ;;
+        --keep-build-tmp)
+            keep_build_tmp=true
+            shift ;;
+        --legacy_hipblas_direct)
+            legacy_hipblas_direct=true
+            shift ;;
         --) shift ; break ;;
         *)  echo "Unexpected command line parameter received: '${1}'; aborting";
             exit 1
@@ -554,22 +591,22 @@ if [[ "${install_dependencies}" == true ]]; then
   install_packages
 
   CMAKE_VERSION=$(cmake --version | grep -oP '(?<=version )[^ ]*' )
-  if [ -z "$CMAKE_VERSION" ] || $(dpkg --compare-versions $CMAKE_VERSION lt 3.16.8); then
+  if [ -z "$CMAKE_VERSION" ] || $(dpkg --compare-versions $CMAKE_VERSION lt 3.25.2); then
       if $update_cmake == true; then
         pushd
         printf "\033[32mBuilding \033[33mcmake\033[32m from source; installing into \033[33m/usr/local\033[0m\n"
-        CMAKE_REPO="https://github.com/Kitware/CMake/releases/download/v3.16.8/"
+        CMAKE_REPO="https://github.com/Kitware/CMake/releases/download/v3.25.2/"
         mkdir -p ${build_dir}/deps && cd ${build_dir}/deps
-        wget -nv ${CMAKE_REPO}/cmake-3.16.8.tar.gz
-        tar -xvf cmake-3.16.8.tar.gz
-        rm cmake-3.16.8.tar.gz
-        cd cmake-3.16.8
+        wget -nv ${CMAKE_REPO}/cmake-3.25.2.tar.gz
+        tar -xvf cmake-3.25.2.tar.gz
+        rm cmake-3.25.2.tar.gz
+        cd cmake-3.25.2
         ./bootstrap --no-system-curl --parallel=16
         make -j16
         sudo make install
         popd
       else
-          echo "hipBLASLt requires CMake version >= 3.16.8 and CMake version ${CMAKE_VERSION} is installed. Run install.sh again with --cmake_install flag and CMake version 3.16.8 will be installed to /usr/local"
+          echo "hipBLASLt requires CMake version >= 3.25.2 and CMake version ${CMAKE_VERSION} is installed. Run install.sh again with --cmake_install flag and CMake version 3.16.8 will be installed to /usr/local"
           exit 2
       fi
   fi
@@ -619,6 +656,10 @@ pushd .
   cmake_common_options="-DAMDGPU_TARGETS=${gpu_architecture}"
   cmake_client_options=""
 
+  if [[ "${legacy_hipblas_direct}" == true ]]; then
+    cmake_common_options="${cmake_common_options} -DLEGACY_HIPBLAS_DIRECT=ON"
+  fi
+
   # build type
   if [[ "${build_release}" == true ]]; then
     mkdir -p ${build_dir}/release/clients && cd ${build_dir}/release
@@ -647,7 +688,7 @@ pushd .
 
   # library type
   if [[ "${build_static}" == true ]]; then
-    cmake_common_options="{cmake_common_options} -DBUILD_SHARED_LIBS=OFF"
+    cmake_common_options="${cmake_common_options} -DBUILD_SHARED_LIBS=OFF"
   fi
 
   # clients
@@ -667,7 +708,7 @@ pushd .
       fi
   fi
 
- if [[ -n "${tensile_fork}" ]]; then
+  if [[ -n "${tensile_fork}" ]]; then
     cmake_common_options="${cmake_common_options} -Dtensile_fork=${tensile_fork}"
   fi
 
@@ -688,8 +729,8 @@ pushd .
     tensile_opt="${tensile_opt} -DBUILD_WITH_TENSILE=OFF"
    else
     tensile_opt="${tensile_opt} -DTensile_LOGIC=${tensile_logic} -DTensile_CODE_OBJECT_VERSION=${tensile_cov}"
-    if [[ ${build_jobs} != $(nproc) ]]; then
-      tensile_opt="${tensile_opt} -DTensile_CPU_THREADS=${build_jobs}"
+    if [[ ${tensile_threads} != $(nproc) ]]; then
+      tensile_opt="${tensile_opt} -DTensile_CPU_THREADS=${tensile_threads}"
     fi
   fi
 
@@ -707,12 +748,24 @@ pushd .
     tensile_opt="${tensile_opt} -DTensile_ASM_DEBUG=ON"
   fi
 
+  if [[ "${enable_gprof}" == true ]]; then
+    if [[ "${build_static}" == false ]]; then
+      printf "'--gprof' can only be enabled for static build.\n"
+      exit 2
+    fi
+    cmake_common_options="${cmake_common_options} -DCMAKE_CXX_FLAGS=-pg -DCMAKE_C_FLAGS=-pg"
+  fi
+
+  if [[ "${keep_build_tmp}" == true ]]; then
+    tensile_opt="${tensile_opt} -DTensile_KEEP_BUILD_TMP=ON"
+  fi
+
   echo $cmake_common_options
   cmake_common_options="${cmake_common_options} ${tensile_opt}"
 
-  compiler="hcc"
+  compiler="amdclang++"
   if [[ "${build_hip_clang}" == true ]]; then
-    compiler="${rocm_path}/bin/hipcc"
+    compiler="${rocm_path}/bin/amdclang++"
   fi
 
   if [[ "${build_clients}" == false ]]; then
