@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@ from . import ClientExecutable
 from . import LibraryIO
 from .TensileInstructions import getGfxName, DataType, getCOVFromParam
 from .Common import globalParameters, pushWorkingPath, popWorkingPath, print1, printExit, CHeader, printWarning, listToInitializer, ClientExecutionLock
-from .SolutionStructs import Problem, ProblemType, ProblemSizesMock, ProblemSizesMockDummy, ActivationArgs, BiasTypeArgs
+from .SolutionStructs import Problem, ProblemType, ProblemSizesMock, ProblemSizesMockDummy, ActivationArgs, BiasTypeArgs, FactorDimArgs
 from .TensileCreateLibrary import copyStaticFiles
 
 import os
@@ -116,7 +116,7 @@ def main( config ):
       biasTypeArgs = ""
 
     activationEnums = [[{'Enum': 'relu'}]]
-
+    factorDimEnums = [0]
     # Reading the activation args from the LibraryClient section in the config YAML.
     # Example: enable relu and gelu activation and using none to run without activation
     #    LibraryClient:
@@ -124,18 +124,26 @@ def main( config ):
     #        - [Enum: none]
     #        - [Enum: gelu]
     #        - [Enum: relu]
+    icacheFlushArgs = [False,]
     if len(config) > 0:
       for lc in config[0:]:
         if "ActivationArgs" in lc:
           activationEnums = lc["ActivationArgs"]
           break
+        if "FactorDimArgs" in lc:
+          factorDimEnums = lc["FactorDimArgs"]
+        if "ICacheFlush" in lc:
+          icacheFlushArgs = lc["ICacheFlush"]
     activationArgs = ActivationArgs(problemType, activationEnums) if problemType["ActivationType"] == 'all' else ""
+    factorDimArgs = FactorDimArgs(problemType, factorDimEnums)
     clientParametersPaths.append(writeClientConfig(
                                   forBenchmark=False,
                                   solutions=None,
                                   problemSizes=problemSizes,
                                   biasTypeArgs=biasTypeArgs,
+                                  factorDimArgs=factorDimArgs,
                                   activationArgs=activationArgs,
+                                  icacheFlushArgs=icacheFlushArgs,
                                   stepName=str(ProblemType(problemType)),
                                   stepBaseDir=globalParameters["WorkingPath"],
                                   newLibrary=newLibrary,
@@ -355,7 +363,7 @@ def checkConstStride(constStrideMap, keyIdx):
   return finalVal
 
 
-def problemSizeParams(problemType, problem):
+def problemSizeParams(problemType, problem, factorDim):
 
     numIndices = len(problemType.indices)
     rv = []
@@ -434,16 +442,25 @@ def problemSizeParams(problemType, problem):
       if problemType.useE:
           rv.append(('e-strides', ",".join(map(str, dstrides))))
     if problemType.useBias:
-      biasstrides = [1, problem.sizes[0], 0]
+      length = problem.sizes[0]
+      err_str = "M"
+      if problemType.sparse:
+        if len(factorDim) > 1:
+          length = max(problem.sizes[0], problem.sizes[1])
+          err_str = "max(M,N)"
+        elif 1 in factorDim:
+          length = problem.sizes[1]
+          err_str = "N"
+      biasstrides = [1, length, 0]
       for sc in problemType.setConstStrideBias:
         index = problemType.indices[sc[0]]
         if type(index) == BatchIndex:
             biasstrides[2] = sc[1]
       if biasstrides[2] == -1:
-        biasstrides[2] = problem.sizes[0]
-      elif biasstrides[2] != 0 and biasstrides[2] < problem.sizes[0]:
-        raise RuntimeError("problem-specified bias stride(%u) must >= M (%u)" % \
-              (biasstrides[2], problem.sizes[0]))
+        biasstrides[2] = length
+      elif biasstrides[2] != 0 and biasstrides[2] < length:
+        raise RuntimeError("problem-specified bias stride(%u) must >= %s (%u)" % \
+              (biasstrides[2], err_str, length))
       rv.append(('bias-strides', ",".join(map(str, biasstrides))))
 
     return rv
@@ -499,7 +516,7 @@ def pruneModeName(mode):
     if mode == 5: return 'Prune0X0X'
     if mode == 6: return 'Prune00XX'
 
-def writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, problemType, sourceDir, codeObjectFiles, resultsFileName, parametersFilePath, libraryFile=None):
+def writeClientConfigIni(problemSizes, biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, problemType, sourceDir, codeObjectFiles, resultsFileName, parametersFilePath, libraryFile=None):
 
     with open(parametersFilePath, "w") as f:
         def param(key, value):
@@ -525,6 +542,8 @@ def writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, problemType
         param('d-type',     problemType.dType.toEnum())
         if problemType.useE:
             param('e-type',     problemType.eType.toEnum())
+        if problemType.outputAmaxD:
+            param('amaxD-type',     problemType.amaxDType.toEnum())
         param('alpha-type', problemType.alphaType.toEnum())
         param('beta-type',  problemType.betaType.toEnum())
         param('f32-xdl-math-op', problemType.f32XdlMathOp.toEnum())
@@ -533,19 +552,29 @@ def writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, problemType
         param('use-bias',   problemType.useBias)
         param('bias-source',   problemType.biasSrcWhiteList[0])
         param('use-e', problemType.useE)
+        param('output-amaxD', problemType.outputAmaxD)
         param('use-scaleAB',   problemType.useScaleAB)
         param('use-scaleCD',   problemType.useScaleCD)
         param('use-scaleAlphaVec',   problemType.useScaleAlphaVec)
         if biasTypeArgs:
           for btype in biasTypeArgs.biasTypes:
             param('bias-type-args',  btype.toEnum())
+        if factorDimArgs:
+          for fdim in factorDimArgs.factorDims:
+            param('factor-dim-args', fdim)
+
+
+        if icacheFlushArgs:
+          for opt in icacheFlushArgs:
+            param('icache-flush-args', opt)
+
         param('sparse',   problemType.sparse)
         param('high-precision-accumulate', problemType.highPrecisionAccumulate)
         param('strided-batched', problemType.stridedBatched)
         param('grouped-gemm', problemType.groupedGemm)
 
         for problem in problemSizes.problems:
-            for key,value in problemSizeParams(problemType, problem):
+            for key,value in problemSizeParams(problemType, problem, factorDimArgs.factorDims):
                 param(key,value)
 
         if activationArgs:
@@ -577,6 +606,8 @@ def writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, problemType
           param("print-tensor-ref",       1)
         if globalParameters["PrintTensorBias"]:
           param("print-tensor-bias",      1)
+        if globalParameters["PrintTensorAmaxD"]:
+          param("print-tensor-amaxd",      1)
         if globalParameters["DumpTensors"]:
           param("dump-tensors",           1)
         if globalParameters["ExitOnFails"] > 1:
@@ -613,9 +644,10 @@ def writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, problemType
 
         param("use-user-args",            globalParameters["UseUserArgs"])
         param("rotating-buffer-size",     globalParameters["RotatingBufferSize"])
+        param("rotating-buffer-mode",     globalParameters["RotatingMode"])
 
 
-def writeClientConfig(forBenchmark, solutions, problemSizes, biasTypeArgs, activationArgs, stepName, stepBaseDir, newLibrary, codeObjectFiles, tileAwareSelection, configBase = "ClientParameters", libraryFile = None):
+def writeClientConfig(forBenchmark, solutions, problemSizes, biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, stepName, stepBaseDir, newLibrary, codeObjectFiles, tileAwareSelection, configBase = "ClientParameters", libraryFile = None):
 
     if tileAwareSelection:
       filename = os.path.join(globalParameters["WorkingPath"], "%s_Granularity.ini"%configBase)
@@ -633,7 +665,7 @@ def writeClientConfig(forBenchmark, solutions, problemSizes, biasTypeArgs, activ
 
     newSolution = next(iter(newLibrary.solutions.values()))
     sourceDir = os.path.join(stepBaseDir, "source")
-    writeClientConfigIni(problemSizes, biasTypeArgs, activationArgs, newSolution.problemType, sourceDir, codeObjectFiles, resultsFileName, filename, libraryFile)
+    writeClientConfigIni(problemSizes, biasTypeArgs, factorDimArgs, activationArgs, icacheFlushArgs, newSolution.problemType, sourceDir, codeObjectFiles, resultsFileName, filename, libraryFile)
 
     return filename
 
@@ -650,11 +682,11 @@ def CreateBenchmarkClientParametersForSizes(libraryRootPath, problemSizes, dataF
       metaDataFilePath = os.path.join(libraryPath, "metadata.yaml")
       if not os.path.exists(metaDataFilePath):
         printExit ("meta data file %s does not exist" % metaDataFilePath)
-      metaData = LibraryIO.readYAML(metaDataFilePath)
+      metaData = LibraryIO.read(metaDataFilePath)
       problemTypeDict = metaData["ProblemType"]
       problemType = ContractionsProblemType.FromOriginalState(problemTypeDict)
 
-    writeClientConfigIni(problemSizes, "", problemType, libraryRootPath, codeObjectFiles, dataFilePath, configFile)
+    writeClientConfigIni(problemSizes, "", "", "", "", problemType, libraryRootPath, codeObjectFiles, dataFilePath, configFile)
 
 
 ################################################################################
