@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -68,7 +68,7 @@ class ProblemType:
                  'useBeta', 'useBias', 'biasSrcWhiteList', 'useE', 'useScaleAB', 'useScaleCD', 'useScaleAlphaVec', 'biasDataTypeWhiteList',
                  'highPrecisionAccumulate', 'useInitialStridesAB', 'useInitialStridesCD', 'stridedBatched', 'groupedGemm',
                  'useGradient', 'activationType', 'activationArgLength', 'activationComputeDataType', 'activationNoGuard',
-                 'sparse', 'f32XdlMathOp', 'supportDeviceUserArguments']
+                 'sparse', 'f32XdlMathOp', 'supportDeviceUserArguments', 'outputAmaxD']
     @classmethod
     def FromOriginalState(cls, d):
         indices = [None]*d['TotalIndices']
@@ -129,7 +129,7 @@ class ProblemType:
 
         rv.transA = bool(d['TransposeA'])
         rv.transB = bool(d['TransposeB'])
-        if 'DataTypeA' in d:
+        if 'DataTypeA' in d: #it will either be set as d['DataType'] or a specified input
             rv.aType = DataType(d['DataTypeA'])
         else:
             rv.aType = srcType
@@ -137,17 +137,23 @@ class ProblemType:
             rv.bType = DataType(d['DataTypeB'])
         else:
             rv.bType = srcType
+
+        if rv.aType.isFloat8BFloat8() or rv.bType.isFloat8BFloat8():
+            rv.aType = DataType("F8")
+            rv.bType = DataType("B8")
+        elif rv.aType.isBFloat8Float8() or rv.bType.isBFloat8Float8():
+            rv.aType = DataType("B8")
+            rv.bType = DataType("F8")
+
         if 'DataTypeE' in d:
             rv.eType = DataType(d['DataTypeE'])
         else:
             rv.eType = dstType
-        # for hybrid 8bit float types, we need to split the type into a_type and b_type
-        if srcType.isFloat8BFloat8():
-            rv.aType = DataType("F8")
-            rv.bType = DataType("B8")
-        elif srcType.isBFloat8Float8():
-            rv.aType = DataType("B8")
-            rv.bType = DataType("F8")
+
+        if 'DataTypeAmaxD' in d:
+            rv.amaxDType = DataType(d['DataTypeAmaxD'])
+        else:
+            rv.amaxDType = computeType
 
         rv.computeInputType = srcType
         rv.cType = dstType
@@ -190,10 +196,11 @@ class ProblemType:
         if 'UseBeta' in d:
             rv.useBeta = d['UseBeta']
 
-        rv.useBias               = False
+        rv.useBias               = 0
         rv.biasDataTypeWhiteList = []
         rv.biasSrcWhiteList = []
         rv.setConstStrideBias = []
+
         if 'UseBias' in d:
             rv.useBias = d['UseBias']
             if 'BiasDataTypeList' in d:
@@ -215,14 +222,18 @@ class ProblemType:
         if 'Gradient' in d:
             rv.useGradient = d["Gradient"]
 
-        rv.useScaleAB = False
+        rv.outputAmaxD = False
+        if 'OutputAmaxD' in d:
+            rv.outputAmaxD = d['OutputAmaxD']
+
+        rv.useScaleAB = ""
         if 'UseScaleAB' in d:
             rv.useScaleAB = d['UseScaleAB']
         rv.useScaleCD = False
         if 'UseScaleCD' in d:
             rv.useScaleCD = d['UseScaleCD']
 
-        rv.useScaleAlphaVec = False
+        rv.useScaleAlphaVec = 0
         if 'UseScaleAlphaVec' in d:
             rv.useScaleAlphaVec = d['UseScaleAlphaVec']
 
@@ -337,6 +348,7 @@ class ProblemType:
                 predicates.append(ProblemPredicate("BetaZero"))
             predicates.append(ProblemPredicate("BiasDataTypeWhiteList", value=self.biasDataTypeWhiteList))
             predicates.append(ProblemPredicate("BiasSrcWhiteList", value=self.biasSrcWhiteList))
+            predicates.append(ProblemPredicate("AmaxDCheck", value=self.outputAmaxD))
             if self.activationType == 'all':
                 exportType = ActivationType.Export.GRADONLY if self.useGradient else ActivationType.Export.NORMAL
                 enumList = [actEnum.capitalize() for actEnum in ActivationType.getEnumStrList(self.activationComputeDataType, exportType=exportType)]
@@ -406,9 +418,6 @@ class ProblemPredicate(Properties.Predicate):
         if key.startswith('Assert'):
             raise RuntimeError("Unknown assertion key: {}".format(key))
 
-        if key == "Fp16AltImpl":
-            return cls("Fp16AltImpl") if value != False else None
-
     @classmethod
     def CompoundPredicates(cls, state, problemType):
         rv = []
@@ -420,8 +429,11 @@ class ProblemPredicate(Properties.Predicate):
             valuepredicates = [];
             valuepredicates.append(state["MacroTile0"])
             valuepredicates.append(state["MacroTile1"])
-            valuepredicates.append(state["NumElementsPerThread"])
-            valuepredicates.append(state["StoreVectorWidth"])
+            valuepredicates.append(state["MIWaveTile"][0]*state["MIWaveTile"][1])
+            if state["NumElementsPerBatchStore"] != 0:
+                valuepredicates.append(int((state["NumElementsPerThread"])/state["NumElementsPerBatchStore"]))
+            else:
+                valuepredicates.append(1)
             valuepredicates.append(state["NumThreads"])
             rv += [cls('SynchronizerSizeCheck', index=0, value=valuepredicates)]
 
@@ -492,8 +504,14 @@ class SizeMapping:
                  'workGroup',
                  'macroTile',
                  'threadTile',
+                 'matrixInstruction',
+                 'grvwA',
+                 'grvwB',
+                 'gwvwC',
+                 'gwvwD',
                  'depthU',
                  'staggerU',
+                 'staggerUMapping',
                  'globalSplitUPGR',
                  'globalSplitU',
                  'staggerStrideShift',
@@ -504,26 +522,34 @@ class SizeMapping:
                  'globalAccumulation',
                  'workspaceSizePerElemC',
                  'workspaceSizePerElemBias',
-                 'activationFused', 'CustomKernelName'
+                 'activationFused',
+                 'CustomKernelName',
+                 'workGroupMappingXCC',
+                 'globalSplitUCoalesced',
+                 'globalSplitUWorkGroupMappingRoundRobin'
                  ]
 
     @classmethod
     def FromOriginalState(cls, d):
-        globalAccum = 2
-        # FIXME: Restore this after the yamls are fixed
-        # globalAccum = 0
-        # if d['GlobalSplitUAlgorithm'] == 'SingleBuffer':
-        #     globalAccum = 1
-        # if d['GlobalSplitUAlgorithm'] == 'MultipleBuffer':
-        #     globalAccum = 2
+        globalAccum = 0
+        if d['GlobalSplitUAlgorithm'] == 'SingleBuffer':
+            globalAccum = 1
+        if d['GlobalSplitUAlgorithm'] == 'MultipleBuffer':
+            globalAccum = 2
         if d['_GlobalAccumulation'] == 'MultipleBufferSingleKernel':
             globalAccum = 3
         return cls(waveNum                  = d['NumThreads'] // d['WavefrontSize'],
                    workGroup                = d['WorkGroup'],
                    macroTile                = cls.ReadOriginalMacroTile(d),
                    threadTile               = d['ThreadTile'],
+                   matrixInstruction        = d['MatrixInstruction'],
+                   grvwA                    = d['GlobalReadVectorWidthA'],
+                   grvwB                    = d['GlobalReadVectorWidthB'],
+                   gwvwC                    = d['StoreVectorWidth'],
+                   gwvwD                    = d['StoreVectorWidth'],
                    workGroupMapping         = d['WorkGroupMapping'],
                    staggerU                 = d['StaggerU'] if 'StaggerU' in d else 0,
+                   staggerUMapping          = d['StaggerUMapping'] if 'StaggerUMapping' in d else 0,
                    depthU                   = d['DepthU'],
                    globalSplitUPGR          = internalParameters["GlobalSplitUPGR"],
                    globalSplitU             = d['GlobalSplitU'],
@@ -535,7 +561,10 @@ class SizeMapping:
                    workspaceSizePerElemC    = d['_WorkspaceSizePerElemC'],
                    workspaceSizePerElemBias = d['_WorkspaceSizePerElemBias'],
                    activationFused          = d['ActivationFused'],
-                   CustomKernelName         = d['CustomKernelName']
+                   CustomKernelName         = d['CustomKernelName'],
+                   workGroupMappingXCC      = d['WorkGroupMappingXCC'],
+                   globalSplitUCoalesced    = d['GlobalSplitUCoalesced'],
+                   globalSplitUWorkGroupMappingRoundRobin = d['GlobalSplitUWorkGroupMappingRoundRobin']
                    )
 
     @classmethod
@@ -550,13 +579,19 @@ class SizeMapping:
             setattr(self, key, value)
 
 class InternalArgsSupport:
-    StateKeys = ['gsu',
-                 'wgm']
+    StateKeys = ['version',
+                 'gsu',
+                 'wgm',
+                 'staggerU',
+                 'useUniversalArgs']
 
     @classmethod
     def FromOriginalState(cls, d):
-        return cls(gsu = d['InternalSupportParams']['SupportUserGSU'],
-                   wgm = d['InternalSupportParams']['SupportCustomWGM'])
+        return cls(version = d['InternalSupportParams']['KernArgsVersion'],
+                   gsu = d['InternalSupportParams']['SupportUserGSU'],
+                   wgm = d['InternalSupportParams']['SupportCustomWGM'],
+                   staggerU = d['InternalSupportParams']['SupportCustomStaggerU'],
+                   useUniversalArgs = d['InternalSupportParams']['UseUniversalArgs'])
 
     def __init__(self, **kwargs):
         for (key, value) in list(kwargs.items()):
