@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -89,7 +89,7 @@ class MatchingLibrary:
             3: Properties.Property("FreeSizeB", index=0),
             1: Properties.Property("BoundSize", index=0)
         }
-        if distance == "Equality":
+        if distance == "Equality" or distance == "GridBased":
             propertyKeys[0] = Properties.Property("BatchSize", index=0)
 
         properties = list([propertyKeys[i] for i in indices if i in propertyKeys])
@@ -175,7 +175,7 @@ class FreeSizeLibrary:
 
 class DecisionTreeLibrary:
     Tag = "DecisionTree"
-    StateKeys = [("type", "tag"), "features", "trees"]
+    StateKeys = [("type", "tag"), "features", "trees", "nullValue"]
 
     @classmethod
     def FromOriginalState(cls, d, solutions):
@@ -183,6 +183,14 @@ class DecisionTreeLibrary:
         origTrees = d["trees"]
 
         trees = []
+        
+        if "fallback" in d:
+            fallbackIndex = d["fallback"]
+            nullValue = SingleSolutionLibrary(solutions[fallbackIndex])
+        else:
+            raise RuntimeError(
+                "DecisionTree has no fallback; likely a legacy model which is not supported."
+            )
 
         for tree in origTrees:
             index = tree["solution"]
@@ -191,7 +199,7 @@ class DecisionTreeLibrary:
             entry = {"tree": tree["tree"], "value": value}
             trees.append(entry)
 
-        return cls(features, trees)
+        return cls(features, trees, nullValue)
 
     @property
     def tag(self):
@@ -205,9 +213,10 @@ class DecisionTreeLibrary:
     def remapSolutionIndices(self, indexMap):
         pass
 
-    def __init__(self, features, trees):
+    def __init__(self, features, trees, nullValue):
         self.features = features
         self.trees = trees
+        self.nullValue = nullValue
 
 
 class ProblemMapLibrary:
@@ -256,8 +265,12 @@ class PredicateLibrary:
             else:
                 self.rows.append(row)
 
-        # Sort to ensure consistent fallback logic.
-        self.rows.sort(key=lambda x: x["predicate"])
+        if self.rows[0]["library"].tag == "Placeholder":
+            # Sort to ensure pure gemm can be search first
+            self.rows.sort(key=lambda x: len(x["library"].filenamePrefix))
+        else:
+            # Sort to ensure consistent fallback logic.
+            self.rows.sort(key=lambda x: x["predicate"])
 
     def remapSolutionIndices(self, indexMap):
         for row in self.rows:
@@ -288,7 +301,8 @@ class MasterSolutionLibrary:
                           origData,
                           origSolutions,
                           solutionClass=Contractions.Solution,
-                          libraryOrder=None):
+                          libraryOrder=None,
+                          placeholderName='TensileLibrary'):
 
         # functions for creating each "level" of the library
         def hardware(d, problemType, solutions, library, placeholderName):
@@ -334,19 +348,6 @@ class MasterSolutionLibrary:
 
             return newLib, placeholderName
 
-        def fp16AltImpl(d, problemType, solutions, library, placeholderName):
-            if d.get("Fp16AltImpl"):
-                predicate = Properties.Predicate(tag="Fp16AltImpl")
-            else:
-                predicate = Properties.Predicate(tag="TruePred")
-            newLib = PredicateLibrary(tag="Problem")
-            newLib.rows.append({"predicate": predicate, "library": library})
-
-            if lazyLibrary and predicate.tag != "TruePred":
-                placeholderName += "_Fp16Alt"
-
-            return newLib, placeholderName
-
         def predicates(d, problemType, solutions, library, placeholderName):
             predicates = problemType.predicates(includeBatch=True, includeType=True)
             predicate = Contractions.ProblemPredicate.And(predicates)
@@ -395,6 +396,37 @@ class MasterSolutionLibrary:
             else:
                 assert 0 and "Unrecognized LibraryType."
 
+            if Common.globalParameters["LazyLibraryLoading"]:
+                placeholderName += '_' + str(problemType.aType) + str(problemType.bType)
+                placeholderName += '_' + str(problemType.cType) + str(problemType.computeInputType)
+                if problemType.activationType != 'none':
+                    if str(problemType.activationType).upper() == 'ALL':
+                        placeholderName += "_A"
+                    else:
+                        placeholderName += "_%s"%str(problemType.activationType).upper()
+                if problemType.useBias:
+                    placeholderName += '_Bias'
+                if problemType.useE:
+                    placeholderName += '_Grad' if problemType.useGradient else '_Aux'
+                if problemType.groupedGemm:
+                    placeholderName += "_GG"
+                else:
+                    placeholderName += "" if problemType.stridedBatched else "_GB" # legacy
+                if problemType.useScaleAB == "Scalar":
+                    placeholderName += '_SAB'
+                elif problemType.useScaleAB == "Vector":
+                    placeholderName += '_SABV'
+                if problemType.useScaleCD:
+                    placeholderName += '_SCD'
+                if problemType.useScaleAlphaVec:
+                    placeholderName += '_SAV'
+                if problemType.sparse:
+                    placeholderName += '_SPB' if problemType.sparse == 2 else '_SPA'
+                if not problemType.f32XdlMathOp.isSingle() and problemType.computeInputType.isSingle():
+                    placeholderName += '_M' + str(problemType.f32XdlMathOp)
+                if problemType.supportDeviceUserArguments:
+                    placeholderName += '_UA'
+
             return library, placeholderName
 
         # end library creation functions
@@ -402,12 +434,12 @@ class MasterSolutionLibrary:
         if libraryOrder is None:
             if Common.globalParameters["LazyLibraryLoading"]:
                 libraryOrder = [
-                    hardware, operationIdentifier, performanceMetric, fp16AltImpl, predicates,
+                    hardware, operationIdentifier, performanceMetric, predicates,
                     placeholder, selection
                 ]
             else:
                 libraryOrder = [
-                    hardware, operationIdentifier, performanceMetric, fp16AltImpl, predicates,
+                    hardware, operationIdentifier, performanceMetric, predicates,
                     selection
                 ]
         #assert libraryOrder[-1] == selection
@@ -415,9 +447,12 @@ class MasterSolutionLibrary:
         lazyLibrary = None
         if placeholder in libraryOrder:
             placeholderIndex = libraryOrder.index(placeholder) + 1
-            lazyLibrary = MasterSolutionLibrary.FromOriginalState(origData, origSolutions,
-                                                                  solutionClass,
-                                                                  libraryOrder[placeholderIndex:])
+            lazyLibrary, placeholderName = \
+                MasterSolutionLibrary.FromOriginalState(origData,
+                                                        origSolutions,
+                                                        solutionClass,
+                                                        libraryOrder[placeholderIndex:],
+                                                        placeholderName)
             libraryOrder = libraryOrder[0:placeholderIndex]
             origSolutions = []
 
@@ -427,7 +462,6 @@ class MasterSolutionLibrary:
 
         # library is constructed in reverse order i.e. bottom-up
         library = None
-        placeholderName = "TensileLibrary"
         placeholderLibrary = None
         for libName in reversed(libraryOrder):
             library, placeholderName = libName(origData, problemType, allSolutions, library,
@@ -441,7 +475,7 @@ class MasterSolutionLibrary:
             rv.lazyLibraries[placeholderName] = lazyLibrary
             placeholderLibrary.filenamePrefix = placeholderName
 
-        return rv
+        return rv, placeholderName
 
     @classmethod
     def BenchmarkingLibrary(cls, solutions):
