@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -48,7 +48,7 @@ namespace Tensile
     class TENSILE_API ContractionProblemParameters
     {
     public:
-        void setGSU(uint8_t gsu)
+        void setGSU(uint16_t gsu)
         {
             m_gsu = gsu;
         }
@@ -58,7 +58,27 @@ namespace Tensile
             return m_gsu;
         }
 
-        void setWgm(uint8_t wgm)
+        void setGSUC(bool gsuc)
+        {
+            m_gsuc = gsuc;
+        }
+
+        bool gsuc() const
+        {
+            return m_gsuc;
+        }
+
+        void setGSUWGMRR(bool gsuwgmrr)
+        {
+            m_gsuwgmrr = gsuwgmrr;
+        }
+
+        bool gsuwgmrr() const
+        {
+            return m_gsuwgmrr;
+        }
+
+        void setWgm(int16_t wgm)
         {
             m_wgm = wgm;
         }
@@ -78,6 +98,16 @@ namespace Tensile
             return m_biasType;
         }
 
+        void setFactorDim(int factorDim)
+        {
+            m_factorDim = factorDim;
+        }
+
+        int factorDim() const
+        {
+            return m_factorDim;
+        }
+
         void setActivationEnum(ActivationType activationEnum)
         {
             m_activationType = activationEnum;
@@ -94,9 +124,12 @@ namespace Tensile
         }
 
     private:
-        uint8_t        m_gsu            = 0; // default value
-        uint8_t        m_wgm            = 0; // default value
+        uint16_t       m_gsu            = 0; // default value
+        bool           m_gsuc           = false; // default value
+        bool           m_gsuwgmrr       = false; // default value
+        int16_t        m_wgm            = 0; // default value
         DataType       m_biasType       = DataType::None;
+        int            m_factorDim      = 0;
         ActivationType m_activationType = ActivationType::None;
     };
 
@@ -236,6 +269,7 @@ namespace Tensile
             SCALEALPHAVEC = 10,
             METADATA      = 11,
             Synchronizer  = 12,
+            AMAXD         = 13,
             TENSOR_COUNT
         };
 
@@ -576,12 +610,17 @@ namespace Tensile
             m_useE = useE;
         }
 
-        void setUseBias(bool useBias)
+        void setOutputAmaxD(bool outputAmaxD)
+        {
+            m_outputAmaxD = outputAmaxD;
+        }
+
+        void setUseBias(int useBias)
         {
             m_useBias = useBias;
         }
 
-        void setUseScaleAB(bool useScaleAB)
+        void setUseScaleAB(std::string useScaleAB)
         {
             m_useScaleAB = useScaleAB;
         }
@@ -591,7 +630,7 @@ namespace Tensile
             m_useScaleCD = useScaleCD;
         }
 
-        void setUseScaleAlphaVec(bool useScaleAlphaVec)
+        void setUseScaleAlphaVec(int useScaleAlphaVec)
         {
             m_useScaleAlphaVec = useScaleAlphaVec;
         }
@@ -601,12 +640,17 @@ namespace Tensile
             return m_useE;
         }
 
-        bool useBias() const
+        bool outputAmaxD() const
+        {
+            return m_outputAmaxD;
+        }
+
+        int useBias() const
         {
             return m_useBias;
         }
 
-        bool useScaleAB() const
+        std::string useScaleAB() const
         {
             return m_useScaleAB;
         }
@@ -616,7 +660,7 @@ namespace Tensile
             return m_useScaleCD;
         }
 
-        bool useScaleAlphaVec() const
+        int useScaleAlphaVec() const
         {
             return m_useScaleAlphaVec;
         }
@@ -638,10 +682,12 @@ namespace Tensile
         void setBias(DataType                       type,
                      size_t                         length,
                      size_t                         stride,
-                     bool                           isOutput = false,
-                     ContractionProblemGemm::TENSOR src      = ContractionProblemGemm::TENSOR::D)
+                     bool                           isOutput  = false,
+                     ContractionProblemGemm::TENSOR src       = ContractionProblemGemm::TENSOR::D,
+                     int                            factorDim = 0)
         {
             setParams().setBiasEnum(type);
+            setParams().setFactorDim(factorDim);
             m_biasSrc = src;
             if(type != DataType::None && m_useBias)
             {
@@ -680,23 +726,23 @@ namespace Tensile
             return m_biasSrc;
         }
 
-        void setScaleA(DataType type)
+        void setScaleA(DataType type, size_t length)
         {
             m_scaleAType = type;
-            if(type != DataType::None && m_useScaleAB)
+            if(type != DataType::None && !m_useScaleAB.empty())
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEA]
-                    = {"scaleA", m_scaleAType, {1}, {1, 1}};
+                    = {"scaleA", m_scaleAType, {length}, {1, length}};
             }
         }
 
-        void setScaleB(DataType type)
+        void setScaleB(DataType type, size_t length)
         {
             m_scaleBType = type;
-            if(type != DataType::None && m_useScaleAB)
+            if(type != DataType::None && !m_useScaleAB.empty())
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEB]
-                    = {"scaleB", m_scaleBType, {1}, {1, 1}};
+                    = {"scaleB", m_scaleBType, {length}, {1, length}};
             }
         }
 
@@ -720,16 +766,26 @@ namespace Tensile
             }
         }
 
-        void setScaleAlphaVec(DataType type, size_t length)
+        void setScaleAlphaVec(DataType type, size_t length, int factorDim = 0)
         {
             m_scaleAlphaVecType = type;
             if(type != DataType::None && m_useScaleAlphaVec)
             {
+                setParams().setFactorDim(factorDim);
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEALPHAVEC]
                     = {"scaleAlphaVec", m_scaleAlphaVecType, {length}, {1, length}};
             }
         }
 
+        void setAmaxD(DataType type, bool isOutput = false)
+        {
+            if(type != DataType::None && m_outputAmaxD)
+            {
+                m_tensors[ContractionProblemGemm::TENSOR::AMAXD] = {"amaxD", type, {1}, {1, 1}};
+                m_tensors[ContractionProblemGemm::TENSOR::AMAXD].setAsOutput(isOutput);
+            }
+        }
+
         void setSynchronizer(DataType type, size_t length)
         {
             if(type != DataType::None)
@@ -829,16 +885,6 @@ namespace Tensile
             return m_deterministicMode;
         }
 
-        void setFp16AltImpl(bool value)
-        {
-            m_fp16AltImpl = value;
-        }
-
-        bool fp16AltImpl() const
-        {
-            return m_fp16AltImpl;
-        }
-
         void setUseGradient(bool value)
         {
             m_useGradient = value;
@@ -943,6 +989,10 @@ namespace Tensile
         {
             return m_tensors[ContractionProblemGemm::TENSOR::BIAS];
         }
+        TensorDescriptor const& amaxd() const
+        {
+            return m_tensors[ContractionProblemGemm::TENSOR::AMAXD];
+        }
         FreeIndices const& freeIndicesA() const
         {
             return m_freeIndicesA;
@@ -1053,20 +1103,25 @@ namespace Tensile
                            double              beta,
                            size_t              workspaceSize);
 
-        static ContractionProblemGemm createDefaultProblem(bool     transA,
-                                                           bool     transB,
-                                                           DataType typeA,
-                                                           DataType typeB,
-                                                           DataType typeC,
-                                                           DataType typeD,
-                                                           DataType alphaType,
-                                                           DataType betaType,
-                                                           DataType typeComputeInput,
-                                                           DataType typeCompute,
-                                                           double   alpha,
-                                                           double   beta,
-                                                           bool     isGroupedGemm,
-                                                           size_t   maxWorkspaceBytes);
+        static ContractionProblemGemm
+            createDefaultProblem(bool                   transA,
+                                 bool                   transB,
+                                 DataType               typeA,
+                                 DataType               typeB,
+                                 DataType               typeC,
+                                 DataType               typeD,
+                                 DataType               typeAlpha,
+                                 DataType               typeBeta,
+                                 DataType               typeComputeInput,
+                                 DataType               typeCompute,
+                                 double                 alpha,
+                                 double                 beta,
+                                 bool                   useBias,
+                                 bool                   useGradient,
+                                 std::vector<DataType>& biasDataTypeWhiteList,
+                                 std::vector<int>&      biasSrcWhiteList,
+                                 bool                   isGroupedGemm,
+                                 size_t                 maxWorkspaceBytes);
 
     private:
         std::string m_sumNames;
@@ -1081,13 +1136,13 @@ namespace Tensile
         bool           m_highPrecisionAccumulate = false;
         bool           m_deterministicMode       = false;
         bool           m_eligibleForPK           = true;
-        bool           m_fp16AltImpl             = false;
         bool           m_useGradient             = false;
         bool           m_useE                    = false;
-        bool           m_useBias                 = false;
-        bool           m_useScaleAB              = false;
+        bool           m_outputAmaxD             = false;
+        int            m_useBias                 = 0;
+        std::string    m_useScaleAB              = "";
         bool           m_useScaleCD              = false;
-        bool           m_useScaleAlphaVec        = false;
+        int            m_useScaleAlphaVec        = 0;
         ActivationType m_activationType          = ActivationType::None;
         bool           m_activationNoGuard       = false;
         int            m_sparse                  = 0;
@@ -1190,11 +1245,12 @@ namespace Tensile
                           unsigned char const* _metadata);
 
         // TODO: Remove this
-        void const* a = nullptr;
-        void const* b = nullptr;
-        void const* c = nullptr;
-        void*       d = nullptr;
-        void*       e = nullptr;
+        void const* a     = nullptr;
+        void const* b     = nullptr;
+        void const* c     = nullptr;
+        void*       d     = nullptr;
+        void*       e     = nullptr;
+        void*       amaxD = nullptr;
 
         void const* const* batchA    = nullptr;
         void const* const* batchB    = nullptr;
